var DataType = {
	NONE: 0,
	LIST: 1,
	SEARCH: 2,
	ITEM: 3
};

var $$connector = null;

function Connector(compleateCallback)
{
	var o = this;
	$$connector = this;
	this.countries = [];
	this.genres = [];
	this.types = ["Фильмы", "Сериалы"];
	this.sorts = ["Популярные сверху", "Рейтинговые сверху", "Новые сверху"];
	this.years = [
		{name: "2000-е", rule: "year:[2000 TO 2009]"},
		{name: "90-е", rule: "year:[1990 TO 1999]"},
		{name: "80-е", rule: "year:[1980 TO 1989]"},
		{name: "70-е", rule: "year:[1970 TO 1979]"},
		{name: "60-е", rule: "year:[1960 TO 1969]"},
		{name: "50-е", rule: "year:[1950 TO 1959]"},
		{name: "40-е", rule: "year:[1940 TO 1949]"},
		{name: "до 40-х", rule: "year:[* TO 1939]"}
		];
	var today = new Date();
	var yr = today.getFullYear();
	for(var y = 2010; y <= yr; y++)
		this.years.unshift({name: ""+y, rule: "year:"+y});
	
	this.rates = [
		{name: "от 9", rule: "rating:[9 TO *]"},
		{name: "8", rule: "rating:[8 TO *]"},
		{name: "7", rule: "rating:[7 TO *]"},
		{name: "6", rule: "rating:[6 TO *]"},
		{name: "5", rule: "rating:[5 TO *]"},
		{name: "4", rule: "rating:[4 TO *]"},
		{name: "3", rule: "rating:[3 TO *]"},
		{name: "2", rule: "rating:[2 TO *]"},
		{name: "1", rule: "rating:[1 TO *]"}
		];
	
	this.compleateCallback = compleateCallback;
	this.displayItemsOnPage = 10;
	
	this.selectedTypeId = 0;
	this.selectedCountryId = -1;
	this.selectedGenreId = -1;
	this.selectedSortId = 0;
	this.selectedYearId = -1;
	this.selectedRateId = -1;
	
	this.searchQuery = "";
	this.selectedItem = null;
	
	this.currentPage = 0;
	this.pagesCount = 0;
	this.hasMorePages = false;
	this.pagerBlocked = false;
	this.navigating = false;
	
	this.OnUpdatedMenu = null;
	this.OnUpdatedContent = null;
	this.OnPlay = null;
	
	this.lastResultType = DataType.NONE;
	this.lastResult = null;

	this.GetGenreById = function(id)
	{
		for(var i = 0; i < this.genres.length; i++)
			if(this.genres[i].id == id)
				return this.genres[i];
		return null;
	};
	
	this.BuildGenres = function(ids)
	{
		ids = ids.split(" ");
		var result = [];
		for(var i = 0; i < ids.length; i++)
		{
			var genre = ids[i].trim();
			if(!genre)
				continue;
			genre = parseInt(genre);
			if(genre === NaN)
				continue;
			var genre = this.GetGenreById(genre);
			if(genre != null)
				result.push(genre);
		}
		if(result.length == 0)
			return "";
		if(result.length > 1)
			result.sort(function(a, b) {return a.ord - b.ord});
		for(var i = 0; i < result.length; i++)
			result[i] = result[i].name;
		return result.join(", ");
	};
	
	this.DuildSerialDataQuery = function()
	{
		var baseUrl = "http://api2.zona.mobi/solr/episode/select/";
		var params = [];
		var value = "episode_key,title,mobi_link_id,release_date";
		params.push({name: "fl", value: value});
		params.push({name: "version", value: "2.2"});
		params.push({name: "start", value: "0"});
		params.push({name: "rows", value: "2147483647"});
		params.push({name: "wt", value: "json"});
		params.push({name: "q", value: "(kinopoisk_id:" + this.selectedItem.id + ")"});
		var query = "";
		for(var i = 0; i < params.length; i++)
			query += (i == 0 ? "?" : "&") + params[i].name + "=" + encodeURIComponent(params[i].value);
		return baseUrl + query;
	}
	
	this.BuildQuery = function()
	{
		var baseUrl = "http://api2.zona.mobi/solr/movie/select/";
		var params = [];
		var value;
		if(this.selectedItem == null)
		{
			value = "id,year,rating,rating_kinopoisk_count,rating_imdb_count,name_rus,name_eng,name_id,mobi_link_id,serial,genreId,country";
			if(this.selectedTypeId == 1 || this.searchQuery)//Сериал / поиск
				value += ",serial_ended,serial_end_year";
		}
		else
		{
			params.push({name: "start", value: "0"});
			value = "rating_imdb,rating_kinopoisk,rating_imdb_count,rating_kinopoisk_count,name_id,name_rus,name_original,description,year,genreId,country,release_date_rus,release_date_int,runtime,id,release_date_hq,persons,mobi_link_id,mobi_link_date,rating,serial,abuse";
			if(this.selectedItem.serial)
				value += ",serial_ended,serial_end_year";
		}
		params.push({name: "fl", value: value});
		if(this.selectedItem == null)
		{
			if(!this.searchQuery)
			{
				switch(this.selectedSortId)
				{
				case 0://Популярные
					value = "popularity desc,seeds desc";
					break;
				case 1://Рейтинговые
					value = "rating desc,rating_count desc";
					break;
				case 2://Новые
					value = "year desc,release_date desc";
					break;
				}
			}
			else
				value = "score desc,popularity desc,seeds desc";
			params.push({name: "sort", value: value});
			params.push({name: "start", value: "" + (this.displayItemsOnPage * this.currentPage)});
		}
		params.push({name: "wt", value: "json"});
		params.push({name: "rows", value: "" + (this.selectedItem == null ? this.displayItemsOnPage : 1)});
		params.push({name: "version", value: "2.2"});
		var rules = [];
		//rules.push("(adult:false)");
		if(this.selectedItem == null)
		{
			if(!this.searchQuery)
			{
				rules.push(this.selectedTypeId == 1 ? "(serial:true)" : "(serial:false)");
				if(this.selectedGenreId > -1)
					rules.push("(genreId:" + this.genres[this.selectedGenreId].id + ")");
				if(this.selectedCountryId > -1)
				{
					var subRules = [];
					var country = this.countries[this.selectedCountryId];
					if(country.ids)
						for(var i = 0; i < country.ids.length; i++)
							subRules.push("(country_id:" + country.ids[i] + ")");
					subRules.push("(country_id:" + country.id + ")");
					if(subRules.length == 1)
						rules.push(subRules[0]);
					else
						rules.push("(" + subRules.join("OR") + ")");
				}
				if(this.selectedYearId > -1)
					rules.push("(" + this.years[this.selectedYearId].rule + ")");
				if(this.selectedRateId > -1)
					rules.push("(" + this.rates[this.selectedRateId].rule + ")");
				value = rules.join("AND");
			}
			else
			{
				rules.push("(name_original:(" + this.searchQuery + "))");
				rules.push("(name_rus_search:(" + this.searchQuery + "))");
				rules.push("(name_eng:(" + this.searchQuery + "))");
				value = "(" + rules.join("OR") + ")";
			}
		}
		else
		{
			rules.push("(name_id:" + this.selectedItem.name_id + ")");
			rules.push("(serial:" + (this.selectedItem.serial ? "true" : "false") + ")");
			value = rules.join("AND");
		}
		params.push({name: "q", value: value});
		var query = "";
		for(var i = 0; i < params.length; i++)
			query += (i == 0 ? "?" : "&") + params[i].name + "=" + encodeURIComponent(params[i].value);
		return baseUrl + query;
	};
	
	this.UpdateInternal = function (lostPage)
	{
		if(lostPage)
			this.currentPage = 0;
		if(this.OnUpdatedMenu != null)
			this.OnUpdatedMenu(this);
		this.UpdateContent();
	};
	
	this.UpdateContent = function ()
	{
		var url = this.BuildQuery();
		alert(url);
		this.pagerBlocked = true;
		this.navigating = true;
		jQuery.getJSON(url, {}, this.SetResults);
	};
	
	this.SetCountriesAndGenres = function (data, textStatus, jqXHR)
	{
		o.countries = data.countries;
		o.genres = data.genres;
		o.genres.sort(function(a, b) {return a.ord - b.ord});
		o.compleateCallback(o);
		o.UpdateInternal(true);
	};
	
	this.FailCountriesAndGenres = function ()
	{
		o.countries = [];
		o.genres = [];
		o.compleateCallback(o);
		o.UpdateInternal(true);
	};
	
	this.SetResults = function (data, textStatus, jqXHR)
	{
		var docs = data.response.docs;
		var found = data.response.numFound;
		var start = data.response.start;
		if(this.selectedItem == null)
		{
			o.hasMorePages = start + docs.length < found;
			o.pagesCount = Math.ceil(found / o.displayItemsOnPage);
		}
		for(var i = 0; i < docs.length; i++)
		{
			var s = (Math.floor(Math.random() * 4) % 4) + 1;
			var id = "" + docs[i].id;
			var subId = id.substring(0, id.length - 3);
			if(!subId)
				subId = "0";
			docs[i].image_url = "http://img" + s + ".zona.ru/images/film_240/" + subId + "/" + id + ".jpg";
			docs[i].genre = o.BuildGenres(docs[i].genreId);
			if(docs[i].rating < 0)
				docs[i].rating += 10;
			if(o.selectedItem != null)
			{
				if(docs[i].persons)
				{
					docs[i].persons = JSON.parse(docs[i].persons);
					docs[i].regis = [];
					docs[i].scene = [];
					docs[i].actor = [];
					for(var ii = 0; ii < docs[i].persons.length; ii++)
					{
						var person = docs[i].persons[ii];
						if(person.r == 4)
						{
							s = (Math.floor(Math.random() * 4) % 4) + 1;
							id = "" + person.i;
							subId = id.substring(0, id.length - 3);
							if(!subId)
								subId = "0";
							person.image = "http://img" + s + ".zona.ru/images/actor/" + subId + "/" + id + ".jpg";
							docs[i].actor.unshift({name: person.name, image: person.image});
						}
						else if(person.r == 3)
							docs[i].scene.unshift(person.name);
						else if(person.r == 1)
							docs[i].regis.unshift(person.name);
					}
					docs[i].scene = docs[i].scene ? docs[i].scene.join(", ") : "";
					docs[i].regis = docs[i].regis ? docs[i].regis.join(", ") : "";
				}
				if(docs[i].runtime)
					docs[i].runtime = {total: docs[i].runtime, hours: Math.floor(docs[i].runtime / 60), minutes: docs[i].runtime % 60};
			}
		}
		if(o.selectedItem == null)
		{
			o.lastResult = docs;
			if(o.searchQuery)
				o.lastResultType = DataType.SEARCH;
			else
				o.lastResultType = DataType.LIST;
		}
		else if(o.selectedItem.serial)
		{
			var mainData = docs ? docs[0] : null;
			
			
			var url = o.DuildSerialDataQuery();
			jQuery.getJSON(url, {}, function(data, textStatus, jqXHR) {
				if(mainData != null)
				{
					var episodes = [];
					for(var i = 0; i < data.response.docs.length; i++)
					{
						var episode = /S(\d+)E(\d+)/.exec(data.response.docs[i].episode_key);
						var season = parseInt(episode[1], 10);
						episode = parseInt(episode[2], 10);
						
						var linkId = data.response.docs[i].mobi_link_id;
						episodes.push({season: season, episode: episode, linkId: linkId});
					}
					episodes.sort(function(a, b)
					{
						if(a.season == b.season)
							return a.episode - b.episode;
						return a.season - b.season;
					});
					var currentSeason = -1;
					var season = null;
					var seasons = [];
					for(var i = 0; i < episodes.length; i++)
					{
						var episode = episodes[i];
						if(currentSeason < episode.season || season == null)
						{
							if(season != null)
								seasons.push(season);
							currentSeason = episode.season;
							season = {season: episode.season, episodes: []};
						}
						season.episodes.push(episode);
					}
					if(season != null)
						seasons.push(season);
					mainData.seasons = seasons;
				}
				o.lastResult = mainData;
				o.lastResultType = DataType.ITEM;
				o.pagerBlocked = false;
				o.navigating = false;
				if(o.OnUpdatedContent != null)
					o.OnUpdatedContent(o);
			});
			return;
		}
		else
		{
			o.lastResult = docs ? docs[0] : null;
			o.lastResultType = DataType.ITEM;
		}
		o.pagerBlocked = false;
		o.navigating = false;
		if(o.OnUpdatedContent != null)
			o.OnUpdatedContent(o);
	};
	
	this.PlayInternal = function(data)
	{
		this.pagerBlocked = false;
		this.navigating = false;
		if(this.OnPlay != null)
			this.OnPlay(this, data);
	};
	
	var url = "http://api2.zona.mobi/content/mobi/countriesAndGenres";
	jQuery.getJSON(url, {}).done(this.SetCountriesAndGenres).fail(this.FailCountriesAndGenres);
};

Connector.prototype.NextPage = function()
{
	if(!this.hasMorePages || this.pagerBlocked || this.selectedItem != null)
		return false;
	this.currentPage++;
	this.UpdateContent();
	return true;
}

Connector.prototype.PrevPage = function()
{
	if(this.currentPage == 0 || this.pagerBlocked || this.selectedItem != null)
		return false;
	this.currentPage--;
	this.UpdateContent();
	return true;
}

Connector.prototype.ListTypes = function()
{
	var result = [];
	for(var i = 0; i < this.types.length; i++)
		result.push(this.types[i]);
	return result;
}

Connector.prototype.ListCountries = function()
{
	var result = [];
	result.push("Все страны");
	for(var i = 0; i < this.countries.length; i++)
		result.push(this.countries[i].name);
	return result;
}

Connector.prototype.ListGenres = function()
{
	var result  = [];
	result.push("Все жанры");
	for(var i = 0; i < this.genres.length; i++)
		result.push(this.genres[i].name);
	return result;
}

Connector.prototype.ListSorts = function()
{
	var result  = [];
	for(var i = 0; i < this.sorts.length; i++)
		result.push(this.sorts[i]);
	return result;
}

Connector.prototype.ListYears = function()
{
	var result  = [];
	result.push("Любой год");
	for(var i = 0; i < this.years.length; i++)
		result.push(this.years[i].name);
	return result;
}

Connector.prototype.ListRates = function()
{
	var result  = [];
	result.push("Любой рейтинг");
	for(var i = 0; i < this.rates.length; i++)
		result.push(this.rates[i].name);
	return result;
}

Connector.prototype.SelectedType = function()
{
	return this.types[this.selectedTypeId];
}

Connector.prototype.SelectedCountry = function()
{
	if(this.selectedCountryId == -1)
		return "Все страны";
	return this.countries[this.selectedCountryId].name;
}

Connector.prototype.SelectedGenre = function()
{
	if(this.selectedGenreId == -1)
		return "Все жанры";
	return this.genres[this.selectedGenreId].name;
}

Connector.prototype.SelectedSort = function()
{
	return this.sorts[this.selectedSortId];
}

Connector.prototype.SelectedYear = function()
{
	if(this.selectedYearId == -1)
		return "Любой год";
	return this.years[this.selectedYearId].name;
}

Connector.prototype.SelectedRate = function()
{
	if(this.selectedRateId == -1)
		return "Любой рейтинг";
	return this.rates[this.selectedRateId].name;
}

Connector.prototype.SelectType = function(id)
{
	if(this.selectedTypeId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedTypeId = id;
	this.UpdateInternal(true);
}

Connector.prototype.SelectCountry = function(id)
{
	--id;
	if(this.selectedCountryId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedCountryId = id;
	this.UpdateInternal(true);
}

Connector.prototype.SelectGenre = function(id)
{
	--id;
	if(this.selectedGenreId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedGenreId = id;
	this.UpdateInternal(true);
}

Connector.prototype.SelectSort = function(id)
{
	if(this.selectedSortId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedSortId = id;
	this.UpdateInternal(true);
}

Connector.prototype.SelectYear = function(id)
{
	--id;
	if(this.selectedYearId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedYearId = id;
	this.UpdateInternal(true);
}

Connector.prototype.SelectRate = function(id)
{
	--id;
	if(this.selectedRateId == id && !this.searchQuery && this.selectedItem == null)
		return;
	this.searchQuery = "";
	this.selectedItem = null;
	this.selectedRateId = id;
	this.UpdateInternal(true);
}

Connector.prototype.ContentType = function()
{
	return this.lastResultType;
}

Connector.prototype.Content = function()
{
	return this.lastResult;
}

Connector.prototype.Pages = function()
{
	return this.pagesCount;
}

Connector.prototype.Page = function()
{
	return this.currentPage + 1;
}

Connector.prototype.IsNavigating = function()
{
	return this.navigating;
}

Connector.prototype.Return = function()
{
	if(this.navigating)
		return;
	if(this.selectedItem != null)
	{
		this.selectedItem = null;
		this.UpdateInternal(false);
		return;
	}
	if(this.searchQuery)
	{
		this.searchQuery = "";
		this.UpdateInternal(true);
	}
}

Connector.prototype.Search = function(query)
{
	if(this.navigating || !query)
		return;
	if(this.searchQuery == query && this.selectedItem == null)
		return;
	this.selectedItem = null;
	this.searchQuery = query;
	this.UpdateInternal(this.searchQuery != query);
}

Connector.prototype.Select = function(nameId)
{
	if(this.navigating || this.selectedItem != null)
		return;
	if(this.lastResultType != DataType.LIST && this.lastResultType != DataType.SEARCH)
		return;
	var selected = null;
	for(var i = 0; i < this.lastResult.length; i++)
		if(this.lastResult[i].name_id == nameId)
		{
			selected = this.lastResult[i];
			break;
		}
	if(selected == null)
		return;
	this.selectedItem = selected;
	this.UpdateInternal(false);
}

Connector.prototype.Play = function(id)
{
	if(this.navigating)
		return;
	this.pagerBlocked = true;
	this.navigating = true;
	//var url="http://www.24video.com/video/json/" + id + "?callback=%24%24connector.PlayInternal";
	var url = "http://zona.mobi/ajax/video/" + id;
	//alert("Play: "+url);
	//$.getScript(url);
	jQuery.getJSON(url, {}, function(data){$$connector.PlayInternal(data)});
}
