var VideoMode = {
	FULL: 0,
	LETTER: 1,
	PANANDSCAN: 2,
	ZOOM: 3
}

function PlayerControl(){
	var o = this;
	var b = $("body");
	
	this.banner = new PlayerBanner();
	
	this.OnPlay = null;
	this.OnStop = null;
	
	this.shown = false;
	this.playing = false;
	this.external = false;
	this.loaded = false;
	
	this.mode = VideoMode.LETTER;
	this.screenWidth = 1280;
	this.screenHeight = 720;
	
	this.uiTimer = null;
	this.uiHeight = 0;
	this.updateTimer = null;
	
	this.steps = [
		{n: "5 секунд", v: 5},
		{n: "15 секунд", v: 15},
		{n: "30 секунд", v: 30},
		{n: "1 минута", v: 60},
		{n: "2 минуты", v: 120},
		{n: "5 минут", v: 300},
		{n: "15 минут", v: 900},
		{n: "30 минут", v: 1800},
		{n: "1 час", v: 3600}
	];
	
	this.currentStep = 2;
	
	b.on("nav_key:stop", function() {
		if(!o.shown || o.external)
			return;
		o.Stop(true);
	});
	
	b.on("nav_key:return", function( e ) {
		if(!o.shown || o.external)
			return;
		e.handled = true;
		e.stopPropagation();
		o.Stop(true);
	});
	
	b.on("nav_key:info", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
	});
	
	b.on("mousemove", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
	});
	
	b.on("nav_key:play", function() {
		if(!o.shown || o.external || o.playing)
			return;
		o.playing = true;
		if(o.loaded)
		{
			Player.play();
			o.BeginUpdateUI();
			o.disableScreenSaver();
		}
		o.ShowUI();
		o.banner.Play(!o.loaded);
	});
	
	b.on("nav_key:pause", function() {
		if(!o.shown || o.external)
			return;
		if(o.playing)
		{
			o.playing = false;
			if(o.loaded)
			{
				Player.pause();
				o.StopUpdateUI();
			}
			o.ShowUI();
			o.banner.Pause(!o.loaded);
			o.enableScreenSaver();
		}
		else
		{
			o.playing = true;
			if(o.loaded)
			{
				Player.play();
				o.BeginUpdateUI();
			}
			o.ShowUI();
			o.banner.Play(!o.loaded);
			o.disableScreenSaver();
		}
	});
	
	b.on("nav_key:enter", function() {
		if(!o.shown || o.external)
			return;
		if(o.playing)
		{
			o.playing = false;
			if(o.loaded)
			{
				Player.pause();
				o.StopUpdateUI();
			}
			o.ShowUI();
			o.banner.Pause(!o.loaded);
			o.enableScreenSaver();
		}
		else
		{
			o.playing = true;
			if(o.loaded)
			{
				Player.play();
				o.BeginUpdateUI();
			}
			o.ShowUI();
			o.banner.Play(!o.loaded);
			o.disableScreenSaver();
		}
	});
	
	b.on("nav_key:red", function() {
		if(!o.shown || o.external)
			return;
		o.mode++;
		if(o.mode > 3)
			o.mode = 0;
		if(o.loaded)
			o.ApplyMode();
		switch(o.mode)
		{
		case VideoMode.FULL:
			o.banner.ModeFull();
			break;
		case VideoMode.LETTER:
			o.banner.ModeLetter();
			break;
		case VideoMode.PANANDSCAN:
			o.banner.ModePanandscan();
			break;
		case VideoMode.ZOOM:
			o.banner.ModeZoom();
			break;
		}
	});
	
	b.on("nav_key:up", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.StepUp();
	});
	
	b.on("nav_key:down", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.StepDown();
	});
	
	b.on("nav_key:rew", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.SeekDown();
	});
	
	b.on("nav_key:left", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.SeekDown();
	});
	
	b.on("nav_key:ff", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.SeekUp();
	});
	
	b.on("nav_key:right", function() {
		if(!o.shown || o.external)
			return;
		o.ShowUI();
		o.SeekUp();
	});
	
	Player.on("ready", function() {
		o.ApplyMode();
		o.banner.StopLoading();
		o.ShowUI();
		o.loaded = true;
	});
	
	Player.on("stop", function() {
		if(!o.shown)
			return;
		o.Stop(false);
	});
	
	Player.on("complete", function() {
		if(!o.shown)
			return;
		o.Stop(true);
	});
	
	Player.on("bufferingBegin", function() {
		o.banner.BeginBuffering();
	});	
	
	Player.on("bufferingEnd", function() {
		if(!o.playing)
		{
			Player.pause();
			o.StopUpdateUI();
			o.banner.Pause();
			o.enableScreenSaver();
		}
		else
		{
			o.banner.Play();
			o.BeginUpdateUI();
			o.disableScreenSaver();
		}
		o.banner.StopBuffering();
	});
	
	if(typeof SB.disableScreenSaver == "function" && typeof SB.enableScreenSaver == "function")
	{
		this.screensaverEnabled = true;
		this.disableScreenSaver = function()
		{
			if(!this.shown || !this.screensaverEnabled)
				return;
			SB.disableScreenSaver();
			this.screensaverEnabled = false;
			alert("Screensaver DISABLED");
		};
		this.enableScreenSaver = function()
		{
			if(!this.shown || this.screensaverEnabled)
				return;
			SB.enableScreenSaver();
			this.screensaverEnabled = true;
			alert("Screensaver ENABLED");
		};
	}
	else
	{
		this.disableScreenSaver = function(){};
		this.enableScreenSaver = function(){};
	}
	
	this.RenderStep = function() {
		var step = "Шаг перемотки " + this.steps[this.currentStep].n;
		this.stepLabel.innerHTML = step;
	};
	
	this.StepUp = function() {
		this.currentStep++;
		if(this.currentStep >= this.steps.length)
			this.currentStep = 0;
		this.RenderStep();
	};
	
	this.StepDown = function() {
		this.currentStep--;
		if(this.currentStep < 0)
			this.currentStep = this.steps.length - 1;
		this.RenderStep();
	};
	
	this.RenderProgress = function(customProgress) {
		var duration = Player.videoInfo.duration;
		var currentTime = Player.videoInfo.currentTime;
		var progress = Player.formatTime(currentTime) + " / " + Player.formatTime(duration);
		var iProgress = currentTime / duration * 100;
		$(this.progressBarSlider).css("width", iProgress+"%");
		this.timeLabel.innerHTML = progress;
	};
	
	this.SeekDown = function() {
		var currentTime = Player.videoInfo.currentTime;
		var step = this.steps[this.currentStep].v;
		currentTime -= step;
		if(currentTime < 0)
			currentTime = 0;
		Player.seek(currentTime);
		this.RenderProgress();
	};
	
	this.SeekUp = function() {
		var duration = Player.videoInfo.duration;
		var currentTime = Player.videoInfo.currentTime;
		var step = this.steps[this.currentStep].v;
		currentTime += step;
		if(currentTime >= duration)
		{
			this.Stop(true);
			return;
		}
		Player.seek(currentTime);
		this.RenderProgress();
	};
	
	this.BeginUpdateUI = function() {
		if(this.updateTimer != null)
			return;
		this.updateTimer = setInterval(function(){o.RenderProgress();}, 500);
	};
	
	this.StopUpdateUI = function() {
		if(this.updateTimer == null)
			return;
		clearInterval(this.updateTimer);
		this.updateTimer = null;
	};
	
	//making ui
	(function(){
		var uiWrapper = document.createElement("div");
		uiWrapper.className = "bottom-control-wrapper";
		uiWrapper = b[0].appendChild(uiWrapper);
		var legend = $(".legend")[0];
		legend = b[0].removeChild(legend);
		legend = uiWrapper.appendChild(legend);
		o.playerControlBox = document.createElement("div");
		o.playerControlBox.className = "player-control-box";
		o.playerControlBox = uiWrapper.appendChild(o.playerControlBox);
		o.uiWrapper = uiWrapper;
		o.uiHeight = $(uiWrapper).height();
		$(uiWrapper).addClass("noplayer");
		o.fantomInput = document.createElement("div");
		o.fantomInput.className = "nav-item nav-player-fantom";
		o.fantomInput.style.position = "absolute";
		o.fantomInput.style.top = "0px";
		o.fantomInput.style.left = "0px";
		o.fantomInput.style.width = "0px";
		o.fantomInput.style.height = "0px";
		o.fantomInput.style.overflow = "hidden";
		o.fantomInput = b[0].appendChild(o.fantomInput);
		$(o.fantomInput).hide();
		o.progressBar = document.createElement("div");
		o.progressBar.className = "player-control-progressbar";
		o.progressBar = o.playerControlBox.appendChild(o.progressBar);
		o.progressBarSlider = document.createElement("div");
		o.progressBarSlider.className = "player-control-progressbar-slider";
		o.progressBarSlider = o.progressBar.appendChild(o.progressBarSlider);
		var textline = document.createElement("div");
		textline.className = "player-control-textline";
		textline = o.playerControlBox.appendChild(textline);
		o.timeLabel = document.createElement("span");
		o.timeLabel.className = "player-control-label";
		o.timeLabel = textline.appendChild(o.timeLabel);
		o.stepLabel = document.createElement("span");
		o.stepLabel.className = "player-control-label";
		o.stepLabel = textline.appendChild(o.stepLabel);
		o.RenderStep();
	})();
	
	this.Stop = function(stopPlayer) {
		if(!this.shown)
			return;
		this.enableScreenSaver();
		if(stopPlayer)
			Player.stop(true);
		this.StopUpdateUI();
		if(!this.external)
			this.ReleaseUI();
		this.shown = false;
		this.playing = false;
		this.external = false;
		this.loaded = false;
		if(this.OnStop != null)
			this.OnStop();
	};
	
	this.ApplyMode = function() {
		var videoWidth = Player.videoInfo.width;
		var videoHeight = Player.videoInfo.height;
		var newWidth = this.screenWidth;
		var newHeight = this.screenHeight;
		switch(this.mode)
		{
		case VideoMode.LETTER:
			newHeight = newWidth / videoWidth * videoHeight;
			if(newHeight > this.screenHeight)
			{
				newHeight = this.screenHeight;
				newWidth = newHeight / videoHeight * videoWidth;
			}
			break;
		case VideoMode.PANANDSCAN:
			newWidth = (newHeight / videoHeight * videoWidth + newWidth) / 2;
			newHeight = newWidth / videoWidth * videoHeight;
			break;
		case VideoMode.ZOOM:
			newHeight = newWidth / videoWidth * videoHeight;
			if(newHeight < this.screenHeight)
			{
				newHeight = this.screenHeight;
				newWidth = newHeight / videoHeight * videoWidth;
			}
			break;
		}
		var newX = (this.screenWidth - newWidth) / 2;
		var newY = (this.screenHeight - newHeight) / 2;
		Player.resize(newX, newY, newWidth, newHeight);
		if(!this.playing)
			Player.pause();
	};
	
	this.TakeUI = function(){
		$(this.fantomInput).show();
		$$nav.save();
		$$nav.current(this.fantomInput);
		$(this.uiWrapper).css("bottom", "-" + this.uiHeight + "px").css("opacity", 0).toggleClass("noplayer", false);
		$$legend.show();
		this.banner.Enable();
	};
	
	this.ReleaseUI = function(){
		$(this.fantomInput).hide();
		$$nav.restore();
		if(this.uiTimer != null)
			clearTimeout(this.uiTimer);
		this.uiTimer = null;
		$(this.uiWrapper).stop(true, false).css("bottom", "").css("opacity", "").toggleClass("noplayer", true);
		$$legend.hide();
		this.banner.Disable();
	};
	
	this.ShowUI = function(){
		if(this.uiTimer != null)
		{
			clearTimeout(this.uiTimer);
			this.uiTimer = null;
		}
		else
			$(this.uiWrapper).stop(true, false).animate({bottom: "0px", opacity: 1}, 300);
		var o = this;
		this.uiTimer = setTimeout(function(){
			o.uiTimer = null;
			$(o.uiWrapper).stop(true, false).animate({bottom: "0px", opacity: 0}, {duration: 4000, complete: function() {
				$(o.uiWrapper).css("bottom", "-" + o.uiHeight + "px").css("opacity", 1);
			}});
		}, 15000);
	};
	
};

PlayerControl.prototype.IsExternal = function()
{
	return this.shown && this.external;
}

PlayerControl.prototype.Play = function(url)
{
	alert("Play: " + url);
	if(this.shown)
		return;
	if(this.OnPlay != null)
		this.OnPlay();
	this.shown = true;
	this.playing = true;
	this.external = false;
	this.loaded = false;
	this.TakeUI();
	$(this.progressBarSlider).css("width", "");
	this.banner.BeginLoading();
	Player.play(url);
	this.disableScreenSaver();
}

PlayerControl.prototype.ForceStop = function()
{
	if(!this.shown)
		return;
	this.Stop(true);
}

PlayerControl.prototype.External = function(status)
{
	if(!this.shown)
		return;
	if(status == this.external)
		return;
	if(status)
	{
		if(this.playing)
		{
			Player.pause();
			this.StopUpdateUI();
			this.enableScreenSaver();
		}
		this.ReleaseUI();
	}
	else
	{
		this.TakeUI();
		if(this.playing)
		{
			Player.play();
			this.BeginUpdateUI();
			this.disableScreenSaver();
			this.ShowUI();
		}
	}
	this.external = status;
}
